import * as React from "react";
import * as Velocity from "velocity-react";
import * as Immutable from "immutable";
import ParMenu from "./ParMenu";
import { createImmutableMap } from "../util/createImmutaleMap";
import Scrollbar from "react-scrollbar";
const formatter = function (data, obj) {
    const { defaultMenuId, idField } = obj;
    return data.map(par => {
        par.selected = false;
        par.children.forEach(sub => {
            const status = defaultMenuId === `${sub[idField]}`;
            sub.selected = status;
            if (status) {
                par.selected = true;
            }
        });
        return par;
    });
};
class NavMenu extends React.PureComponent {
    constructor(prop) {
        super(prop);
        this.changeImmutableData = (callback) => {
            let selectStringIndex = "";
            this.setState((pre) => {
                const { immutableData, selected } = pre;
                const { data, selected: _selected } = callback(immutableData, selected);
                selectStringIndex = _selected;
                return {
                    immutableData: data,
                    selected: _selected,
                };
            }, () => {
                const { clickBack, data, } = this.props;
                if (clickBack) {
                    const indexArr = selectStringIndex.split(",");
                    const node = data[+indexArr[0]].children[+indexArr[1]];
                    clickBack(node);
                }
            });
        };
        this.state = this.initState(this.props);
    }
    static getDerivedStateFromProps(nextProps, preState) {
        if (nextProps.data !== preState.preData) {
            const data = formatter(nextProps.data, {
                defaultMenuId: nextProps.defaultMenuId,
                idField: nextProps.idField
            });
            return {
                preData: nextProps.data,
                immutableData: Immutable.fromJS(data),
                selected: nextProps.defaultMenuId || "",
            };
        }
        else if (nextProps.init !== preState.preInit) {
            const data = formatter(nextProps.data, {
                defaultMenuId: nextProps.init.id,
                idField: nextProps.idField
            });
            return {
                immutableData: Immutable.fromJS(data),
                preInit: nextProps.init
            };
        }
        else {
            return null;
        }
    }
    initState(props) {
        const { defaultMenuId, data, idField, textField, urlField, iconField, init } = props;
        const newData = formatter(data, { idField: idField, defaultMenuId: defaultMenuId });
        return {
            selected: defaultMenuId || "",
            immutableData: Immutable.fromJS(newData),
            fieldObj: createImmutableMap({
                id: idField,
                text: textField,
                url: urlField,
                icon: iconField,
            }),
            preData: data,
            preInit: init,
        };
    }
    getParMenu() {
        const { immutableData, fieldObj } = this.state;
        const { idField, expand } = this.props;
        return immutableData.map((node, index) => {
            return (React.createElement(ParMenu, { key: node.get(idField), node: node, expand: expand, fieldObj: fieldObj, parIndex: index, changeData: this.changeImmutableData }));
        });
    }
    render() {
        const parItem = this.getParMenu();
        const { width, children, expand } = this.props;
        const menuCom = expand ? (React.createElement(Scrollbar, { horizontal: false },
            React.createElement("ul", { className: "g-menu" }, parItem))) : (React.createElement("ul", { className: "g-menu" }, parItem));
        return (React.createElement(Velocity.VelocityComponent, { duration: 300, animation: { width: expand ? width || 250 : 70 } },
            React.createElement("div", { className: "g-slideMenu " + (!expand ? "expand" : "") },
                children,
                menuCom)));
    }
}
NavMenu.defaultProps = {
    textField: "name",
    idField: "id",
    iconField: "icon",
    urlField: "url",
    width: 250
};
export default NavMenu;
