import * as React from "react";
import TrItem from './TrItem';
import { SvgIcon } from '../my-icon/index';
class TabView extends React.PureComponent {
    constructor() {
        super(...arguments);
        this.state = {};
        this.colHeadRef = React.createRef();
        this.tabBodyRef = React.createRef();
        this.scrollFn = (e) => {
            if (!e.currentTarget.classList.contains('action-body')) {
                return;
            }
            const { changeScrollTop, viewIndex } = this.props;
            const dom = e.currentTarget;
            const left = dom.scrollLeft;
            const colHead = this.colHeadRef.current;
            colHead.scrollLeft = left;
            changeScrollTop(dom.scrollTop, viewIndex);
        };
        this.wheelFn = (e) => {
            e.preventDefault();
            const { changeScrollTop, viewIndex } = this.props;
            const deltay = e.deltaY;
            const top = this.tabBodyRef.current.scrollTop;
            const distance = top + deltay;
            this.tabBodyRef.current.scrollTop = distance;
            changeScrollTop(distance, viewIndex);
        };
        this.makeSign = (e) => {
            e.preventDefault();
            const type = e.type;
            const dom = e.currentTarget;
            if (type === "mouseenter") {
                dom.classList.add('action-body');
            }
            else if (type === "mouseleave") {
                dom.classList.remove('action-body');
            }
        };
        this.checkAll = (e) => {
            const dom = e.currentTarget;
            const status = dom.dataset.status;
            this.props.changeState(status, 'checkPar');
        };
    }
    overBox() {
        const tbodyDom = this.tabBodyRef.current;
        const { config: { forzen } } = this.props;
        if (!forzen && tbodyDom.scrollHeight > tbodyDom.clientHeight) {
            const child = this.colHeadRef.current.firstElementChild;
            child.style.paddingRight = "18px";
        }
    }
    getCheckAll() {
        const { data } = this.props;
        const allChecked = data.every(val => {
            const status = val.get('checked');
            return status;
        });
        const hasCheck = data.some(val => val.get('checked'));
        const status = allChecked ? "checkbox-marked" : hasCheck ? 'checkbox-has-selected' : "checkbox-blank";
        return (React.createElement("span", { onClick: this.checkAll, "data-status": allChecked },
            React.createElement(SvgIcon, { className: status })));
    }
    gethead() {
        const { config: { child: cols }, } = this.props;
        const tds = cols.map(val => {
            const { text, width, field } = val;
            const txt = field === "check" ? this.getCheckAll() : text;
            return (React.createElement("th", { style: { width: width, }, className: "td-border", key: field }, txt));
        });
        return (React.createElement("div", { className: "tab-head" },
            React.createElement("table", null,
                React.createElement("thead", null,
                    React.createElement("tr", null, tds)))));
    }
    createColgroup() {
        const { config: { child: cols } } = this.props;
        const arr = cols.map(val => {
            const { width, field } = val;
            const style = width ? { width } : undefined;
            return (React.createElement("col", { style: style, key: field }));
        });
        return (React.createElement("colgroup", null, arr));
    }
    componentDidMount() {
        const { config, setTabBodyDom, viewIndex } = this.props;
        const dom = this.tabBodyRef.current;
        if (!dom) {
            return;
        }
        if (!config.forzen) {
            this.overBox();
        }
        setTabBodyDom(dom, viewIndex);
        const { current } = this.tabBodyRef;
        if (config.forzen) {
            current.addEventListener("wheel", this.wheelFn);
        }
    }
    componentWillUnmount() {
        const { config } = this.props;
        if (config.forzen) {
            this.tabBodyRef.current.removeEventListener("wheel", this.wheelFn);
        }
    }
    getBody() {
        const { data, fixObj, fixObj: { idField }, config, viewIndex, changeState, startIndex } = this.props;
        const { child: cols } = config;
        const trs = data.map((val, index) => {
            const id = val.get(idField);
            const isMainView = viewIndex === 0;
            return (React.createElement(TrItem, { key: id, node: val, index: `${index + 1 + startIndex}`, changeState: changeState, cols: cols, isMainView: isMainView, fixObj: fixObj }));
        });
        const colgroup = this.createColgroup();
        const fn = !config.forzen ? this.scrollFn : undefined;
        const makeSignFn = !config.forzen ? this.makeSign : undefined;
        return (React.createElement("div", { className: "tab-body-main", onScroll: fn, onMouseEnter: makeSignFn, onMouseLeave: makeSignFn, ref: this.tabBodyRef },
            React.createElement("table", null,
                colgroup,
                React.createElement("tbody", null, trs))));
    }
    getMainBody() {
        return (React.createElement("div", { className: "tab-body-wrap" }, this.getBody()));
    }
    render() {
        const { config: { width, forzen }, config } = this.props;
        const styleObj = width && forzen ? { width } : undefined;
        const className = config.forzen ? "tab-view-fix" : "tab-view-scroll";
        return (React.createElement("div", { className: `${className} tab-viewBody`, style: styleObj },
            React.createElement("div", { className: "tab-head-wrap", ref: this.colHeadRef }, this.gethead()),
            this.getMainBody()));
    }
}
export default TabView;
